@@ -1,7 +1,7 @@
 /*
  * loading skins
  *
- * Copyright (C) 2014-2022 Mateusz Viste
+ * Copyright (C) 2014-2023 Mateusz Viste
  *
  * MIT License
  *
@@ -364,6 +364,13 @@ void skin_free(struct spritesstruct *sprites) {
   if (sprites->saved) SDL_DestroyTexture(sprites->saved);
   if (sprites->loaded) SDL_DestroyTexture(sprites->loaded);
   if (sprites->nosave) SDL_DestroyTexture(sprites->nosave);
-  for (x = 0; x < 256; x++) if (sprites->font[x]) SDL_DestroyTexture(sprites->font[x]);
+  for (x = 0; x < 256; x++) {
+    /* skip font['_'] because it may be used as a placeholder pointer for many
+     * glyphs, so I will explicitely free it afterwards */
+    if (sprites->font[x] && sprites->font[x] != sprites->font['_']) {
+      SDL_DestroyTexture(sprites->font[x]);
+    }
+  }
+  if (sprites->font['_']) SDL_DestroyTexture(sprites->font['_']);
   free(sprites);
 }
