@@ -3,7 +3,7 @@
  *
  * MIT LICENSE
  *
- * Copyright (C) 2014-2022 Mateusz Viste
+ * Copyright (C) 2014-2023 Mateusz Viste
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -329,9 +329,10 @@ static size_t loadfile2mem(char *file, unsigned char **memptr) {
 
 /* load levels from a file, and put them into an array of up to maxlevels levels */
 int sok_loadfile(struct sokgame **gamelist, int maxlevels, char *gamelevel, unsigned char *memptr, size_t filelen, char *comment, int maxcommentlen) {
-  int loadres, errflag = 0;
+  int errflag = 0;
   unsigned short level;
   unsigned char *allocptr = NULL;
+  struct sokgame *game = NULL;
   if (gamelevel != NULL) {
     filelen = loadfile2mem(gamelevel, &allocptr);
     memptr = allocptr;
@@ -350,45 +351,42 @@ int sok_loadfile(struct sokgame **gamelist, int maxlevels, char *gamelevel, unsi
     if (ungzptr == NULL) return(ERR_UNABLE_TO_OPEN_FILE);
   }
 
-  for (level = 0;; level++) { /* iterate to load games sequentially from the file */
+  for (level = 0; !errflag; level++) { /* iterate to load games sequentially from the file */
     /* puts("loading level.."); */
-    if (level + 1 >= maxlevels) {
-      level--;
-      errflag = ERR_TOO_MANY_LEVELS_IN_SET;
-      break;
-    }
-    gamelist[level] = sok_allocgame();
-    if (gamelist[level] == NULL) {
-      level--;
+    game = sok_allocgame();
+    if(!game) {
       errflag = ERR_MEM_ALLOC_FAILED;
       break;
     }
 
     /* call loadlevelfromfile */
-    loadres = loadlevelfromfile(gamelist[level], &memptr, (level == 0) ? comment : NULL, maxcommentlen);
-
-    if (loadres < 0) { /* error loading level data */
-      if (level == 0) errflag = loadres;
-      sok_freegame(gamelist[level]);
-      level--;
+    errflag = loadlevelfromfile(game, &memptr, (level == 0) ? comment : NULL, maxcommentlen);
+    if (errflag < 0) {
+      if (level)
+        errflag = 0;
+      break;
+    }
+    if (level >= maxlevels) {
+      errflag = ERR_TOO_MANY_LEVELS_IN_SET;
       break;
     }
 
     /* write the level num and load the solution (if any) */
-    gamelist[level]->level = level + 1;
-    gamelist[level]->solution = solution_load(gamelist[level]->crc32, "dat");
-    /* if end of file reached, stop now */
-    if (loadres > 0) break;
+    game->level = level + 1;
+    game->solution = solution_load(game->crc32, "dat");
+    gamelist[level] = game;
+    game = NULL;
   }
 
+  sok_freegame(game);
   if (allocptr != NULL) free(allocptr);
 
-  if (errflag != 0) {
-    sok_freefile(gamelist, level + 1);
+  if (errflag < 0) {
+    sok_freefile(gamelist, level);
     return(errflag);
   }
 
-  return(level + 1);
+  return(level);
 }
 
 /* reloads solutions for all levels in a list */
